Attribute VB_Name = "modWindowsDateFormat"

' **********************************************************************************************************
' Module:      	modWindowsDateFormat
' Author:      	Philipp Stiefel (phil@codekabinett.com)
' Purpose:     	Retrieve the Date Format configured in the Windows Regional Settings.
' Further info: http://codekabinett.com/page.php?Theme=10&Lang=2#date-format-windows-regional-settings-vba-api-sample
' Usage rights: You are free to use this class module in any of your applications free of charge, 
'				as long as this notice remains unchanged.
' **********************************************************************************************************

Option Compare Database
Option Explicit

Public Enum SettingContext
    UserDefault = &H400
    SystemDefault = &H800
End Enum

Public Enum DateFormat
    ShortDate = &H1
    LongDate = &H2
    YearMonth = &H8
    ' MonthDay     = ????       ' New with Windows 10 - Haven't got the header file handy to look it up
End Enum



#If VBA7 Then
    Private Declare PtrSafe Function EnumDateFormatsA Lib "kernel32" (ByVal lpDateFmtEnumProc As LongPtr, ByVal LCID As Long, ByVal dwFlags As Long) As Boolean
    Private Declare PtrSafe Function lstrlenA Lib "kernel32.dll" (ByVal lpString As LongPtr) As Long
    Private Declare PtrSafe Sub CopyMemory Lib "kernel32.dll" Alias "RtlMoveMemory" (ByVal Destination As LongPtr, ByVal Source As LongPtr, ByVal Length As Long)
#Else
    Private Declare Function EnumDateFormatsA Lib "kernel32" (ByVal lpDateFmtEnumProc As Long, ByVal LCID As Long, ByVal dwFlags As Long) As Boolean
    Private Declare Function lstrlenA Lib "kernel32.dll" (ByVal lpString As Long) As Long
    Private Declare Sub CopyMemory Lib "kernel32.dll" Alias "RtlMoveMemory" (ByVal Destination As Long, ByVal Source As Long, ByVal Length As Long)
#End If

#If VBA7 Then
Private Function StringFromPointer(ByVal pointerToString As LongPtr) As String
#Else
Private Function StringFromPointer(ByVal pointerToString As Long) As String
#End If

    Dim tmpBuffer()    As Byte
    Dim byteCount      As Long
    Dim retVal         As String
 
    byteCount = lstrlenA(pointerToString)
    
    If byteCount > 0 Then
  
        ' Resize the buffer as required
        ReDim tmpBuffer(0 To byteCount - 1) As Byte
        
        ' Copy the bytes from pointerToString to out tmpBuffer
        Call CopyMemory(VarPtr(tmpBuffer(0)), pointerToString, byteCount)
    End If
 
    ' Convert Buffer to string
    retVal = StrConv(tmpBuffer, vbUnicode)
    
    StringFromPointer = retVal

End Function

Private m_dateFormat As String

#If VBA7 Then
Private Function EnumDateFormatsProc(ByVal lpDateFormatString As LongPtr) As Boolean
#Else
Private Function EnumDateFormatsProc(ByVal lpDateFormatString As Long) As Boolean
#End If
    m_dateFormat = StringFromPointerA(lpDateFormatString)
    EnumDateFormatsProc = True
End Function

Public Function GetDateFormat(Optional ByVal format As DateFormat = DateFormat.ShortDate, Optional ByVal context As SettingContext = SettingContext.UserDefault) As String

    Dim apiRetVal As Boolean
    m_dateFormat = ""
    
    apiRetVal = EnumDateFormatsA(AddressOf EnumDateFormatsProc, context, format)

    If apiRetVal Then
        GetDateFormat = m_dateFormat
    End If

End Function



' **********************************************************************************************************
' Module:      	modWindowsDateFormat
' Author:      	Philipp Stiefel (phil@codekabinett.com)
' Purpose:     	Retrieve the Date Format configured in the Windows Regional Settings.
' Further info: http://codekabinett.com/page.php?Theme=10&Lang=2#date-format-windows-regional-settings-vba-api-sample
' Usage rights: You are free to use this class module in any of your applications free of charge, 
'				as long as this notice remains unchanged.
' **********************************************************************************************************

